package com.fr.plugin.third.party.jsdbabbe.http;

import com.fanruan.api.log.LogKit;
import com.fanruan.api.util.StringKit;
import com.fr.decision.fun.impl.BaseHttpHandler;
import com.fr.general.IOUtils;
import com.fr.json.JSONObject;
import com.fr.plugin.third.party.jsdbabbe.Utils;
import com.fr.plugin.third.party.jsdbabbe.config.CustomDataConfig;
import com.fr.third.org.apache.commons.codec.digest.DigestUtils;
import com.fr.third.org.apache.http.impl.client.CloseableHttpClient;
import com.fr.third.springframework.web.bind.annotation.RequestMethod;
import com.fr.web.utils.WebUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.Base64;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 *
 */
public class CustomConfigHttpHandler extends BaseHttpHandler {

    @Override
    public RequestMethod getMethod() {
        return RequestMethod.POST;
    }

    @Override
    public String getPath() {
        return "/jsdbabbe/push/message";
    }

    @Override
    public boolean isPublic() {
        return true;
    }

    @Override
    public void handle(HttpServletRequest req, HttpServletResponse res) throws Exception {
        res.setContentType("application/json; charset=utf-8");
        String reqContent = IOUtils.inputStream2String(req.getInputStream());
        if (StringKit.isEmpty(reqContent)) {
            LogKit.error("消息待办推送流程:请求内容为空");
            WebUtils.printAsJSON(res, getFailureJson("请求内容为空"));
            return;
        }

        String url = req.getHeader("Process-Url");
        if (StringKit.isEmpty(url)) {
            url = WebUtils.getHTTPRequestParameter(req, "process_url");
        }
        if (StringKit.isEmpty(url)) {
            LogKit.error("消息待办推送流程:流程地址为空");
            WebUtils.printAsJSON(res, getFailureJson("流程地址为空"));
            return;
        }
        LogKit.info("消息待办推送流程地址:" + url + "\n请求内容:\n" + reqContent);
        String resContent = pushMessage(url, reqContent);
        if (StringKit.isEmpty(reqContent)) {
            WebUtils.printAsJSON(res, getFailureJson("请求内容出错"));
            return;
        }
        WebUtils.printAsJSON(res, getResultJson("success", resContent));
    }

    private String pushMessage(String url, String reqContent) throws IOException {
        //reqContent = handleStr(reqContent);
        CloseableHttpClient httpclient = Utils.createHttpClient(url);
        Base64.Encoder encoder = Base64.getEncoder();
        String auth = CustomDataConfig.getInstance().getUsername() + ":" + CustomDataConfig.getInstance().getPassword();
        String basicAuth = "Basic " + (encoder.encodeToString(auth.getBytes(StandardCharsets.UTF_8)));
        Map<String, String> headers = new HashMap<>();
        headers.put("key", CustomDataConfig.getInstance().getKey());
        LogKit.info("消息待办推送发起流程验证内容:\n" + headers);
        String result = Utils.createHttpPostContent(httpclient, url, reqContent, basicAuth, "application/json; charset=utf-8", headers);
        return result;
    }


    private JSONObject getSuccessJson() {
        JSONObject json = getResultJson("success", "");
        return json;
    }

    private JSONObject getFailureJson(String content) {
        JSONObject json = getResultJson("failure", content);
        return json;
    }

    private JSONObject getResultJson(String status, String content) {
        JSONObject json = new JSONObject();
        json.put("status", status);
        json.put("content", new JSONObject(content));
        return json;
    }


    private String handleStr(String value) {
        if (StringKit.isEmpty(value)) {
            return "";
        }
        String result = value;
        result = result.trim();
        result = result.replaceAll("\n", "");
        result = result.replaceAll("\r", "");
        result = result.trim();
        return result;
    }


}
